///最终结果：rail，需要一个数组保存组件，需要每个组件实现keepalive

import 'package:flutter/material.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: const HomePage(title: 'Save States in NavigationRail'),
    );
  }
}

class HomePage extends StatefulWidget {
  const HomePage({Key? key, this.title}) : super(key: key);

  final String? title;

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  int _selectedPageIndex = 0;
  late List<Widget> _pages;
  late PageController _pageController;

  @override
  void initState() {
    super.initState();

    _selectedPageIndex = 0;

    _pages = [
      const FirstPage(),
      const SecondPage(),
    ];

    _pageController = PageController(initialPage: _selectedPageIndex);
  }

  @override
  void dispose() {
    _pageController.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    var width = MediaQuery.of(context).size.width;
    bool _isExtended = false;
    if (width > 600) _isExtended = true;
    return Scaffold(
      appBar: AppBar(
        title: const Text('Save States in NavigationRail'),
      ),
      body: Row(
        children: <Widget>[
          Column(children: <Widget>[
            Expanded(
              child: NavigationRail(
                minWidth: 56,
                extended: _isExtended,
                selectedIndex: _selectedPageIndex,
                onDestinationSelected: (int index) {
                  _selectedPageIndex = index;
                  _pageController.jumpToPage(index);
                },

                //leading: const Center(child: Text('leading')),
                //trailing: const Center(child: Text('trailing')),
                //groupAlignment: 1,
                labelType: NavigationRailLabelType.none,
                destinations: const <NavigationRailDestination>[
                  NavigationRailDestination(
                    icon: Icon(Icons.favorite_border),
                    selectedIcon: Icon(Icons.favorite),
                    label: Text('First'),
                  ),
                  NavigationRailDestination(
                    icon: Icon(Icons.bookmark_border),
                    selectedIcon: Icon(Icons.book),
                    label: Text('Second'),
                  ),
                ],
              ),
            ),
          ]),
          const VerticalDivider(thickness: 1, width: 1),
          // This is the main content.
          Expanded(
              child: PageView(
            controller: _pageController,
            //The following parameter is just to prevent
            //the user from swiping to the next page.
            physics: const NeverScrollableScrollPhysics(),
            children: _pages,
          ))
        ],
      ),
    );
  }
}

class FirstPage extends StatefulWidget {
  const FirstPage({Key? key}) : super(key: key);

  @override
  _FirstPageState createState() => _FirstPageState();
}

class _FirstPageState extends State<FirstPage>
//with AutomaticKeepAliveClientMixin<Home>
{
  @override
  Widget build(BuildContext context) {
    //super.build(context);
    return const Center(child: Text('Home'));
  }

  //@override
  //bool get wantKeepAlive => true;
}

class SecondPage extends StatefulWidget {
  const SecondPage({Key? key, this.title}) : super(key: key);
  final String? title;

  @override
  _SecondPageState createState() => _SecondPageState();
}

class _SecondPageState extends State<SecondPage>
    with AutomaticKeepAliveClientMixin<SecondPage> {
  int _counter = 0;

  void _incrementCounter() {
    setState(() {
      _counter++;
    });
  }

  @override
  bool get wantKeepAlive => true;

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return Scaffold(
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            const Text(
              'Second Page',
              style: TextStyle(fontWeight: FontWeight.bold, fontSize: 25.0),
            ),
            const Text(
              'You have pushed the button this many times:',
            ),
            Text(
              '$_counter',
              style: const TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: 35.0,
                  color: Colors.blue),
            ),
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: _incrementCounter,
        tooltip: 'Increment',
        child: const Icon(Icons.add),
      ), // This trailing comma makes auto-formatting nicer for build methods.
    );
  }
}
