import 'package:flutter/material.dart';
import '../../irouter.dart';
import 'screens/first.dart';
import 'screens/second.dart';

void main() {
  runApp(const App());
}

class App extends StatefulWidget {
  const App({Key? key}) : super(key: key);

  @override
  _AppState createState() => _AppState();
}

class _AppState extends State<App> {
  final _navigatorKey = GlobalKey<NavigatorState>();
  late final IRouter _routeState;
  late final IRouterDelegate _routerDelegate;
  late final IRouteParser _routeParser;
  late final List<Page<dynamic>> _pages;

  @override
  void initState() {
    _routeParser = IRouteParser(
      routes: {
        '/': (BuildContext context, Map<String, dynamic> params) =>
            const FirstPage(),
        '/second': (context, params) => const SecondPage(),
      },
      initialRoute: '/',
    );

    _routeState = IRouter(_routeParser);
    const _scaffoldKey = ValueKey('App scaffold');

    _pages = [
      const MaterialPage<void>(
        key: _scaffoldKey,
        child: HomePage(),
      ),
    ];

    _routerDelegate = IRouterDelegate(
      routeState: _routeState,
      navigatorKey: _navigatorKey,
      builder: (context) =>
          IRootNavigator(navigatorKey: _navigatorKey, pages: _pages),
    );

    super.initState();
  }

  @override
  Widget build(BuildContext context) => IRouter.provider(
        _routeState,
        MaterialApp.router(
          routerDelegate: _routerDelegate,
          routeInformationParser: _routeParser,
          theme: ThemeData(
            pageTransitionsTheme: const PageTransitionsTheme(
              builders: {
                TargetPlatform.android: FadeUpwardsPageTransitionsBuilder(),
                TargetPlatform.iOS: CupertinoPageTransitionsBuilder(),
                TargetPlatform.linux: FadeUpwardsPageTransitionsBuilder(),
                TargetPlatform.macOS: CupertinoPageTransitionsBuilder(),
                TargetPlatform.windows: FadeUpwardsPageTransitionsBuilder(),
              },
            ),
          ),
        ),
      );

  @override
  void dispose() {
    _routeState.dispose();
    _routerDelegate.dispose();
    super.dispose();
  }
}

class HomePage extends StatefulWidget {
  const HomePage({Key? key, this.title}) : super(key: key);

  final String? title;

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  IRouter? routeState;
  String? userId;

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    //_pageController.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    routeState ??= IRouter.of(context, listen: true);
    final _selectedIndex = routeState!.getPageId();

    debugPrint('build:home');
    // var width = MediaQuery.of(context).size.width;
    // bool _isExtended = false;
    // if (width > 600) _isExtended = true;

    return Scaffold(
      // appBar: AppBar(
      //   title: const Text('use IRouter in BottomNavigation'),
      // ),
      body: ITabNavigator(
        urls: const ['/', '/second'],
        index: _selectedIndex,
        tabId: 'ipage',
      ),
      bottomNavigationBar: BottomNavigationBar(
        items: const [
          BottomNavigationBarItem(
            icon: Icon(Icons.home),
            label: 'First',
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.search),
            label: 'Second',
          ),
        ],
        selectedItemColor: Colors.purple,
        //unselectedItemColor: Colors.purple,
        showSelectedLabels: true,
        showUnselectedLabels: false,
        currentIndex: _selectedIndex,
        onTap: (index) {
          IRouter.push(
              context, routeState!.tabs['ipage']![index].currentUrl, index);
          setState(() {});
        },
      ),
    );
  }
}
